package transportgraph;

import java.io.Serializable;

/**
 * Description of an attribute by name and type, list attributes are lists of
 * the corresponding type, protected attributes are for internal use only and
 * protected from the name space of standard attributes.
 */
public class Attribute implements Serializable {

    /** since KNIME 2.5.0 and visone 2.6.4 . */
    private static final long serialVersionUID = 1L;
    /** Attribute name. */
    private final String m_name;
    /**
     * Attribute type. Equals name of corresponding {@link TransportDataType}
     * enum.
     */
    private final String m_type;
    /** True, if value is a list of type <code>type</code>. */
    private boolean m_isList = false;
    /** True, if value is for system purposes. */
    private boolean m_isProtected = false;

    /**
     * Create new attribute description.
     *
     * @param name
     *            name of attribute
     * @param type
     *            type of attribute, usually name of corresponding
     *            {@link TransportDataType} enum, for unknown types use
     *            different name
     */
    public Attribute(final String name, final String type) {
        this(name, type, false, false);
    }

    /**
     * Create new attribute description.
     *
     * @param name
     *            name of attribute
     * @param type
     *            type of attribute
     */
    public Attribute(final String name, final TransportDataType type) {
        this(name, type, false, false);
    }

    /**
     * Create new attribute description for list attribute.
     *
     * @param name
     *            name of attribute
     * @param type
     *            type of attribute
     * @param isList
     *            true, if value is a list of type
     * @param isProtected
     *            true, if attribute is used for system purposes
     */
    public Attribute(final String name, final String type,
            final boolean isProtected, final boolean isList) {
        m_name = name;
        m_type = type;
        m_isList = isList;
        m_isProtected = isProtected;
    }

    /**
     * Create new attribute description for list attribute.
     *
     * @param name
     *            name of attribute
     * @param type
     *            type of attribute
     * @param isList
     *            true, if value is a list of type
     * @param isProtected
     *            true, if attribute is used for system purposes
     */
    public Attribute(final String name, final TransportDataType type,
            final boolean isProtected, final boolean isList) {
        this(name, type.name(), isProtected, isList);
    }

    /**
     * @return the name
     */
    public String getName() {
        return m_name;
    }

    /**
     * Type equals name of corresponding {@link TransportDataType} enum.
     *
     * @return the type
     */
    public String getType() {
        return m_type;
    }

    /**
     * @return <code>true</code> if the attribute is a list
     */
    public boolean isList() {
        return m_isList;
    }

    /**
     * @return <code>true</code> if the attribute is protected
     */
    public boolean isProtected() {
        return m_isProtected;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return "attribute: name=\"" + m_name + "\" type=\"" + m_type + "\" "
                + (m_isProtected ? "protected " : "")
                + (m_isList ? "list" : "");
    }

}