package transportgraph;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map.Entry;

/**
 * Simple graph description with the sole purpose of transportation over network
 * socket.
 * 
 * @author uwe
 * 
 */
public class TransportGraph implements Serializable {
    /** since KNIME 2.5.0 and visone 2.6.4 . */
    private static final long serialVersionUID = 1L;
    /** Source node index in edge array. */
    public static final int EDGE_INDEX_SOURCE = 0;
    /** Target node index in edge array. */
    public static final int EDGE_INDEX_TARGET = 1;
    /** Index of edge id in edge array. */
    public static final int EDGE_INDEX_ID = 2;
    /** Is edge directed? (name of protected edge attribute) */
    public static final String DIRECTED_ATTRIBUTE =
            "directed_transport_attribute";
    /**
     * List of protected node attributes (string list), that assign nodes to
     * partitions.
     */
    public static final String NODE_PARTITION_ATTRIBUTES =
            "NODE_PARTITION_ATTRIBUTES";

    /** Name of the graph (to show user). */
    public static final String GRAPH_NAME = "GRAPH_NAME";

    /**
     * List of protected edge attributes (string list), that assign edges to
     * partitions.
     */
    public static final String EDGE_PARTITION_ATTRIBUTES =
            "EDGE_PARTITION_ATTRIBUTES";

    /** Array of node ids. */
    private String[] m_nodeIDs;
    /** Array of edges described by [source, target, edge]-id. */
    private String[][] m_edges;

    /**
     * Graph attributes.
     */
    private final HashMap<Attribute, Object> m_graphAttributes =
            new HashMap<Attribute, Object>();

    /** Map of node attributes. */
    private final HashMap<Attribute, Object[]> m_nodeAttributes =
            new HashMap<Attribute, Object[]>();

    /**
     * Edge attributes.
     * 
     */
    private final HashMap<Attribute, Object[]> m_edgeAttributes =
            new HashMap<Attribute, Object[]>();

    /**
     * Create empty transport graph without nodes/edges.
     */
    public TransportGraph() {
        // empty
    }

    /**
     * Create transport graph.
     * 
     * @param nodeIDs
     *            array of nodes (unique id only)
     * @param edges
     *            array of edges [sourceID,targetID, edgeID]
     */
    public TransportGraph(final String[] nodeIDs, final String[][] edges) {
        m_nodeIDs = nodeIDs;
        m_edges = edges;

    }

    /**
     * Add node attribute.
     * 
     * @param attribute
     *            description of this attribute
     * @param values
     *            array of attribute values (same order as {@link #getNodeIDs()}
     *            )
     */
    public void addNodeAttribute(final Attribute attribute,
            final Object[] values) {
        m_nodeAttributes.put(attribute, values);
    }

    /**
     * Create new edge attribute.
     * 
     * @param attribute
     *            description of attribute
     * @param values
     *            array of attribute values (in order of {@link #getEdges()})
     */
    public void addEdgeAttribute(final Attribute attribute,
            final Object[] values) {
        m_edgeAttributes.put(attribute, values);
    }

    /**
     * @return "standard" (neither list, nor protected) node attributes
     */
    public HashMap<Attribute, Object[]> getNodeAttributes() {
        final HashMap<Attribute, Object[]> ret =
                new HashMap<Attribute, Object[]>();
        for (final Entry<Attribute, Object[]> e : m_nodeAttributes.entrySet()) {
            if (e.getKey().isProtected()) {
                continue;
            }
            ret.put(e.getKey(), e.getValue());
        }
        return ret;
    }

    /** @return "standard" (neither list, nor protected) edge attributes */
    public HashMap<Attribute, Object[]> getEdgeAttributes() {
        final HashMap<Attribute, Object[]> ret =
                new HashMap<Attribute, Object[]>();
        for (final Entry<Attribute, Object[]> e : m_edgeAttributes.entrySet()) {
            if (e.getKey().isProtected()) {
                continue;
            }
            ret.put(e.getKey(), e.getValue());
        }
        return ret;
    }

    /** @return "standard" (neither list, nor protected) graph attributes */
    public HashMap<Attribute, Object> getGraphAttributes() {
        final HashMap<Attribute, Object> ret = new HashMap<Attribute, Object>();
        for (final Entry<Attribute, Object> e : m_graphAttributes.entrySet()) {
            if (e.getKey().isProtected()) {
                continue;
            }
            ret.put(e.getKey(), e.getValue());
        }
        return ret;
    }

    /**
     * @return protected (including list) node attributes
     */
    public HashMap<Attribute, Object[]> getProtectedNodeAttributes() {
        final HashMap<Attribute, Object[]> ret =
                new HashMap<Attribute, Object[]>();
        for (final Entry<Attribute, Object[]> e : m_nodeAttributes.entrySet()) {
            if (e.getKey().isProtected()) {

                ret.put(e.getKey(), e.getValue());
            }
        }
        return ret;
    }

    /** @return protected (including list) edge attributes */
    public HashMap<Attribute, Object[]> getProtectedEdgeAttributes() {
        final HashMap<Attribute, Object[]> ret =
                new HashMap<Attribute, Object[]>();
        for (final Entry<Attribute, Object[]> e : m_edgeAttributes.entrySet()) {
            if (e.getKey().isProtected()) {

                ret.put(e.getKey(), e.getValue());
            }
        }
        return ret;
    }

    /** @return protected (including list) graph attributes */
    public HashMap<Attribute, Object> getProtectedGraphAttributes() {
        final HashMap<Attribute, Object> ret = new HashMap<Attribute, Object>();

        for (final Entry<Attribute, Object> e : m_graphAttributes.entrySet()) {
            if (e.getKey().isProtected()) {

                ret.put(e.getKey(), e.getValue());
            }
        }
        return ret;
    }

    /**
     * @param nodeIDs
     *            array of node IDs
     */
    public void setNodeIDs(final String[] nodeIDs) {
        m_nodeIDs = nodeIDs;
    }

    /**
     * @param edges
     *            array of edges as [sourceID, targetID, edgeID]
     */
    public void setEdges(final String[][] edges) {
        m_edges = edges;
    }

    /**
     * Create new graph attribute.
     * 
     * @param attribute
     *            description of attribute
     * @param value
     *            value of attribute
     */
    public void addGraphAttribute(final Attribute attribute,
            final Object value) {
        m_graphAttributes.put(attribute, value);
    }

    /**
     * @return array of node IDs
     */
    public String[] getNodeIDs() {
        return m_nodeIDs;
    }

    /**
     * @return array of edges [sourceID,targetID, edgeID]
     */
    public String[][] getEdges() {
        return m_edges;
    }

}
